import pika
import json
from time import strftime
import datetime
import uuid

from barking_owl.busaccess import BusAccess

class GlobalShutdown():

    def __init__(self,address='localhost',exchange='barkingowl', \
            uid=str(uuid.uuid4()), url_parameters=None, DEBUG=False):

        self.address = address
        self.exchange = exchange
        self.uid = uid
        self.url_parameters = url_parameters        
        self._DEBUG = DEBUG

        #setup message bus
        #self.reqcon = pika.BlockingConnection(pika.ConnectionParameters(host=address))
        #self.reqchan = self.reqcon.channel()
        #self.reqchan.exchange_declare(exchange=exchange,type='fanout')
        #result = self.reqchan.queue_declare(exclusive=True)
        #queue_name = result.method.queue
        #self.reqchan.queue_bind(exchange=exchange,queue=queue_name)
        #self.reqchan.basic_consume(self.reqcallback,queue=queue_name,no_ack=True)

        #self.respcon = pika.BlockingConnection(pika.ConnectionParameters(
        #                                                   host=self.address))
        #self.respchan = self.respcon.channel()
        #self.respchan.exchange_declare(exchange=self.exchange,type='fanout')

        self.bus_access = BusAccess(
            uid = self.uid,
            address = self.address,
            exchange = self.exchange,
            heartbeat_interval = 30,
            url_parameters = self.url_parameters,
            DEBUG = self._DEBUG,
        )

        self.bus_access.set_callback(
            callback = self._reqcallback,
        )

        self.bus_access.stop_listening()
        #self.bus_access.tsleep(1)

        if self._DEBUG == True:
            print "GlobalShutdown.__init__(): init complete."

    def shutdown(self):
        """
        shutdown() sends the global shutdown command to the message bus.
        """

        if self._DEBUG == True:
            print "GlobalShutdown.shutdown(): sending shutdown command" 

        message = {
            'shutdown_datetime': str(datetime.datetime.now())
        }
        #payload = {
        #    'command': 'global_shutdown',
        #    'sourceid': self.uid,
        #    'destinationid': 'broadcast',
        #    'message': packet
        #}
        #jbody = json.dumps(payload)
        #self.respchan.basic_publish(exchange=self.exchange,routing_key='',body=jbody)

        self.bus_access.send_message(
            command = 'global_shutdown',
            destination_id = 'broadcast',
            message = message,
        )

        if self._DEBUG == True:
            print "GlobalShutdown.shutdown(): shutdown message sent successfully."

    def _reqcallback(self,payload): #ch,method,properties,body):
        pass

if __name__ == '__main__':

    print 'Sending Global Shutdown Broadcast ...'

    globalshutdown = GlobalShutdown(
        address='localhost',
        exchange='barkingowl',
        #uid = ,
        #url_parameters = ,
        DEBUG = True,
    )

    globalshutdown.shutdown()

    print 'Exiting.'



import datetime

import pika
import json
import uuid
import time

import threading

class ReceiveThread(threading.Thread):

    def __init__(self,
                 uid=str(uuid.uuid4()),
                 address="localhost",
                 exchange="barkingowl",
                 heartbeat_interval=30,
                 url_parameters=None,
                 DEBUG=False):

        threading.Thread.__init__(self,name="ReceiveThread : %s" % uid)

        self.uid = uid
        self.address = address
        self.exchange = exchange
        self.heartbeat_interval = heartbeat_interval
        self.url_parameters = url_parameters
        self._DEBUG = DEBUG

        self.reqcon = None
        self.reqchan = None
        self._callback = None
        self.consuming = False

    def set_callback(self, callback):
        self._callback = callback

    def run(self):

        if self.url_parameters == None:
            self.reqcon = pika.BlockingConnection(
                pika.ConnectionParameters(
                    host=self.address,
                    heartbeat_interval=self.heartbeat_interval,
                )
            )
        else:
            self.reqcon = pika.BlockingConnection(
                self.url_parameters
            )

        self.reqchan = self.reqcon.channel()
        self.reqchan.exchange_declare(
            exchange=self.exchange,
            type='fanout'
        )
        result = self.reqchan.queue_declare(exclusive=True)
        queue_name = result.method.queue
        self.reqchan.queue_bind(
            exchange=self.exchange,
            queue=queue_name
        )
        self.reqchan.basic_consume(
            self._req_callback,
            queue=queue_name,
            no_ack=True
        )

        if self._DEBUG == True:
            print "ReceiveThread.run(): starting message consuming ..."

        self.consuming = True

        self.reqchan.start_consuming()

    def _req_callback(self, ch, method, properties, body):

        try:
            response = json.loads(body)
            if self._callback != None:
                self._callback(response)
                if self._DEBUG == True:
                    print "BusAccess.req_callback(): Call back called successfully."
        except Exception, e:
            print  "BusAccess.ReceiveThread._req_callback(): error: {0}".format(e)

    def stop_listening(self):
        try:
            if self._DEBUG == True:
                print "ReceiveThread.stop_listening(): Stopping message consuming ..."
            self.reqchan.basic_cancel(nowait=True)
            self.reqchan.stop_consuming()
            #self.reqcon.close()
            if self._DEBUG == True:
                print "ReceiveThread.stop_listening(): Message consuming stopped." 

        except Exception, e:
            if self._DEBUG == True:
                print "ReceiveThread.stop_listening(): ERROR: {0}".format(e)
    def sleep(self, duration):
        if not self.reqchan == None:
            if self._DEBUG == True:
                print "ReceiveThread.sleep(): Sleeping ..."
            self.reqcon.sleep(duration)

    def ready(self):
        _ready = False
        if self.reqcon != None and self.reqchan != None and self.consuming == True:
            _ready = (self.reqcon.is_open and self.reqchan.is_open)
        return _ready

class TransmitThread(threading.Thread):

    def __init__(self,
                 uid=str(uuid.uuid4()),
                 address="localhost",
                 exchange="barkingowl",
                 heartbeat_interval=30,
                 url_parameters=None,
                 DEBUG=False):

        threading.Thread.__init__(self,name="TransmitThread : %s" % uid)

        self.uid = uid
        self.address = address
        self.exchange = exchange
        self.heartbeat_interval = heartbeat_interval
        self.url_parameters = url_parameters
        self._DEBUG = DEBUG

        self.respcon = None
        self.respchan = None

        self._started = False

    def run(self):
        if self.url_parameters == None:
            self.respcon = pika.BlockingConnection(
                pika.ConnectionParameters(
                    host=self.address,
                    heartbeat_interval=self.heartbeat_interval,
                )
            )
        else:
            self.respcon = pika.BlockingConnection(
                self.url_parameters
            )
        self.respchan = self.respcon.channel()
        self.respchan.exchange_declare(
            exchange=self.exchange,
            type='fanout'
        )

        self._started = True

    def send_message(self, command, destination_id, message):
        if self._started == False:
            return

        success = False
        try:
            payload = {
                'command': command,
                'source_id': self.uid,
                'destination_id': destination_id,
                'message': message,
            }
            jbody = json.dumps(payload)
            self.respchan.basic_publish(
                exchange = self.exchange,
                routing_key = '',
                body = jbody,
                mandatory = False,
                immediate = False,
            )
            if self._DEBUG == True:
                print "BusAccess.send_message(): Message sent successfully to message bus."
                #print "BusAccess.send_messsage(): payload: {0}".format(payload)
            success = True
        except Exception, e:
            if self._DEBUG == True:
                print "BusAccess.TransmitThread.send_message(): ERROR: {0}".format(e)

    def sleep(self, duration):
        if not self.respchan == None:
            if self._DEBUG == True:
                print "TransmitThread.sleep(): Sleeping ..."
            self.respcon.sleep(duration)

    def ready(self):
        return self._started

class BusAccess(object):

    def __init__(self,
                 uid=str(uuid.uuid4()),
                 address="localhost",
                 exchange="barkingowl",
                 heartbeat_interval=30,
                 url_parameters=None,
                 DEBUG=False):

        self.uid = uid
        self.address = address
        self.exchange = exchange
        self.heartbeat_interval = heartbeat_interval
        self.url_parameters = url_parameters
        self._DEBUG = DEBUG

        if self._DEBUG == True:
            print "BusAccess.__init__(): starting transmit thread ..." 
        self._transmit_thread = TransmitThread(
            uid = self.uid,
            address = self.address,
            exchange = self.exchange,
            heartbeat_interval = self.heartbeat_interval,
            url_parameters = self.url_parameters,
            DEBUG = self._DEBUG,
        )
        self._transmit_thread.start()
        if self._DEBUG == True:
            print "BusAccess.__init__(): transmit thread started successfully."        

        
        if self._DEBUG == True:
            print "BusAccess.__init__(): starting receive thread ..."
        self._receive_thread = ReceiveThread(
            uid = self.uid,
            address = self.address,
            exchange = self.exchange,
            heartbeat_interval = self.heartbeat_interval,
            url_parameters = self.url_parameters,
            DEBUG = self._DEBUG,
        )
        self._receive_thread.start()
        if self._DEBUG == True:
            print "BusAccess.__init__(): receive thread started successfully."

        # wait until we're ready
        while self._receive_thread.ready() == False or \
                self._transmit_thread.ready() == False:
            pass
        

        if self._DEBUG == True:
            print "BusAccess.__init__(): init complete."

    def set_callback(self, callback):
        self._receive_thread.set_callback(callback)

    def send_message(self, command, destination_id, message):
        self._transmit_thread.send_message(command, destination_id, message) 

    def stop_listening(self):
        self._receive_thread.stop_listening()

    def tsleep(self, duration):
        self._transmit_thread.sleep(duration)

    def rsleep(self,duration):
        self._receive_thread.sleep(duration)




import pika
import json
import uuid
from time import strftime
import time
import threading
import datetime

from scraper import Scraper

from barking_owl.busaccess import BusAccess

import logging
logging.basicConfig()

def log(text, DEBUG=False):
    if DEBUG == True:
        print "[{0}] {1}".format(str(datetime.datetime.now()),text)

class ScraperWrapper(object): #threading.Thread):

    def __init__(self,
                 address='localhost',
                 exchange='barkingowl',
                 heartbeat_interval=30,
                 url_parameters=None,
                 broadcast_interval=5,
                 uid=str(uuid.uuid4()),
                 DEBUG=False):

        #threading.Thread.__init__(self,name="ScraperWrapper : %s" % uid)

        self.uid = str(uuid.uuid4())
        self.address = address
        self.exchange = exchange
        self.heartbeat_interval = heartbeat_interval
        self.url_parameters = url_parameters
        
        self.broadcast_interval = broadcast_interval
        self.uid = uid
        self._DEBUG=DEBUG
 
        self.scraper = Scraper(
            DEBUG = self._DEBUG,
        )
        self.scraping = False
        self.scraper_thread = None

        self.stopped = False

        self.bus_access = BusAccess(
            uid = self.uid,
            address = self.address,
            exchange = self.exchange,
            heartbeat_interval = self.heartbeat_interval,
            url_parameters = self.url_parameters,
            DEBUG = self._DEBUG,
        )
        self.bus_access.set_callback(
            callback = self._reqcallback,
        )

        #threading.Timer(self.interval, self.broadcast_available).start()
        #threading.Timer(self.interval, self.broadcast_simple_status).start()

        #self.broadcast_status()

        #log( "ScraperWrapper.__init__(): Scraper Wrapper INIT complete.", self.DEBUG )

    #def run(self):
    def start(self):
        self.scraper.set_callbacks(
            start_callback = self.scraper_started_callback,
            finished_callback = self.scraper_finished_callback,
            found_doc_callback = self.scraper_broadcast_document_callback,
            new_url_callback = None,
            bandwidth_limit_callback = None,
            memory_limit_callback = None,
            error_callback = None,
        )

        self.broadcast_status()

    def stop(self):
        self.bus_access.stop_listening()
        self.scraper.stop()
        self.stopped = True

    def reset_scraper(self):
        self.scraper.reset()

    def broadcast_status(self):
        
        while not self.scraping and not self.stopped:

            if self._DEBUG == True:
                print "ScraperWrapper.broadcast_status() sending status pulse ..."

            if self.scraping == False and self.scraper._data['working'] == False:
                packet = {
                    'available_datetime': str(datetime.datetime.now())
                }
                self.bus_access.send_message(
                    command = 'scraper_available',
                    destination_id = 'broadcast',
                    message = packet,
                )
            packet = {
                'working': self.scraper._data['working'],
                'seen_url_count': len(self.scraper._data['seen_urls']),
                'document_count': len(self.scraper._data['documents']),
                'url_data': self.scraper._data['url_data'],
                'status_datetime': str(datetime.datetime.now())
            }
            self.bus_access.send_message(
                command = 'scraper_status',
                destination_id = 'broadcast',
                message = packet,
            )

            if self._DEBUG == True:
                print "ScraperWrapper.broadcast_status() status sent successfully."

            if not self.stopped:
                #threading.Timer(self.interval, self.broadcast_available).start()
                self.bus_access.tsleep(self.broadcast_interval)

    def broadcast_simple_status(self):
        if self.scraper._data['url_data'] == {}:
            targeturl = None
        else:
            targeturl = self.scraper._data['url_data']['target_url']
        packet = {
            'working': self.scraper._data['working'],
            'url_count': len(self.scraper._data['seen_urls']),
            'bad_url_count': len(self.scraper._data['bad_urls']),
            'target_url': targeturl,
            'status_datetime': str(datetime.datetime.now())
        }
        self.bus_access.send_message(
            command = 'scraper_available',
            destination_id = 'broadcast',
            message = packet,
        )

    def scraper_finished_callback(self, _data):
        self.bus_access.send_message(
            command = 'scraper_finished',
            destination_id = 'broadcast',
            message = _data,
        )

    def scraper_started_callback(self, _data):
        self.bus_access.send_message(
            command = 'scraper_started',
            destination_id = 'broadcast',
            message = _data,
        )

    def scraper_broadcast_document_callback(self, _data, document):
        self.bus_access.send_message(
            command = 'scraper_found_document', #payload['command'],
            destination_id = 'broadcast', #payload['destination_id'],
            message = document, #payload['message'],
        )

    def _scraperstart(self):
        if self._DEBUG == True:
            print "Starting scraper ..."
        documents = self.scraper.start()
        if self._DEBUG == True:
            print "Scraper complete."
            print documents
        self.scraping = False

    def _reqcallback(self,payload): #ch,method,properties,body):
        try:
            response = payload
           
            if self._DEBUG == True:
                print "ScraperWrapper()._reqcallback(): new message: {0}".format(response)
 
            if response['command'] == 'url_dispatch':
                if response['destination_id'] == self.uid:
                    if self.scraping == False:
                        self.scraper.set_url_data(response['message'])
                        #log( "ScraperWrapper._reqcallback(): Launching scraper thread ...", self.DEBUG )
                        self.scraping = True
                        self.scraper_thread = threading.Thread(target=self._scraperstart)
                        self.scraper_thread.start()
                        #self._scraperstart()
                        #log( "ScraperWrapper._reqcallback(): ... Scraper launched successfully.", self.DEBUG )

            #elif response['command'] == 'scraper_finished':
            #    if response['source_id'] == self.scraper.uid:
            #        self.scraping = False

            elif response['command'] == 'get_status':
                self.broadcast_status()

            elif response['command'] == 'get_status_simple':
                self.broadcast_simple_status()

            elif response['command'] == 'reset_scraper':
                if response['destination_id'] == self.uid:
                    self.resetscraper()

            elif response['command'] == 'shutdown':
                if response['destination_id'] == self.uid:
                    #log( "ScraperWrapper._reqcallback(): [{0}] Shutting Down Recieved".format(self.uid), self.DEBUG ) 
                    self.stop()

            elif response['command'] == 'global_shutdown':
                #log( "ScraperWrapper._reqcallback(): Global Shutdown Recieved", self.DEBUG )
                self.stop()
        except Exception, e:
            print "ScraperWrapper._reqcallback(): ERROR: {0}".format(str(e))



import daemon

from scraperwrapper import ScraperWrapper

def StartScraper(address='localhost',exchange='barkingowl'):
    
    #with daemon.DaemonContext():
    if True:
        try: 
            sw = ScraperWrapper(
                address = address,
                exchange = exchange,
                DEBUG = True,
            )
    
            data = sw.start()

            if not data == None:
                print data['documents']
        except Exception, e:
            print "ERROR: {0}".format(e)

if __name__ == '__main__':

    print ("Launching BarkingOwl Scraper as daemon process.\n"
           "To shutdown scraper, issue (global) shutdown command.\n")

    StartScraper()



from busaccess import BusAccess
import uuid

def callback(payload):
    print "New Payload:"
    print "---------------------------"
    print payload
    print "\n\n"
    if payload['command'] == 'global_shutdown':
        global ba
        ba.stop_listening()

ba = BusAccess(uid=uuid.uuid4())
ba.set_callback(callback)
#ba.start_listening()



import daemon

import datetime
from dispatcher import Dispatcher

def StartDispatcher():

    #with daemon.DaemonContext():
    #    try:
    if True:
        if True:
            dispatcher = Dispatcher(
                address='localhost',
                exchange='barkingowl',
                self_dispatch=False,
                DEBUG=True
            )

            url = {
                'target_url': "http://timduffy.me/",
                'doc_types': [
                'application/pdf',
                    #'*',
                ],
                'title': "TimDuffy.Me",
                'description': "Tim Duffy's Personal Website",
                'max_link_level': 1,
                'creation_datetime': str(datetime.datetime.now()),
                'allowed_domains': [
                ],
                'sleep_time': 0, # 1 secon
            }

            urls = []
            urls.append(url)

            print "Seting URLs ..."
            dispatcher.set_urls(urls)
            print "Done."

            # note: blocking
            print "Starting dispatcher ..."
            dispatcher.start()
            print "Done."

        #except Exception, e:
        #    print "Error: {0}".format(e)

if __name__ == '__main__':

    print ("Launching BarkingOwl Dispatcher as daemon process.\n"
           "To shutdown dispatcher, issue (global) shutdown command.\n")

    StartDispatcher()



import pika
import json
import uuid
from time import strftime
import time
import threading

import datetime

from barking_owl.busaccess import BusAccess

class Dispatcher():

    def __init__(self,
                 address='localhost',
                 exchange='barkingowl',
                 self_dispatch=True,
                 uid=str(uuid.uuid4()),
                 url_parameters=None,
                 broadcast_interval=5,
                 DEBUG=False):

        self.address = address
        self.exchange = exchange
        self.self_dispatch = self_dispatch
        self.uid = uid
        self.url_parameters = url_parameters
        self.broadcast_interval = 5 # 5 seconds
        
        self._DEBUG = DEBUG

        self.exiting = False

        self.current_url_index = 0

        self.urls = []
        self.scrapers = []

        self.bus_access = BusAccess(
            uid = self.uid,
            address = self.address,
            exchange = self.exchange,
            url_parameters = self.url_parameters,
            DEBUG = self._DEBUG,
        )

        self.bus_access.set_callback(
            callback = self._reqcallback,
        )

        #self.broadcast_status()

    def set_urls(self,urls):
        if self._DEBUG == True:
            print "Dispatcher.set_urls(): loading url data"
        for i in range(0,len(urls)):
            urls[i]['start_datetime'] = None
            urls[i]['finish_datetime'] = None
            urls[i]['runs'] = []
        self.current_url_index = 0
        self.urls = urls

    def get_next_url_index(self):

        print "\n"
        print self.urls
        print "\n"

        url_index = -1
        for i in range(0,len(self.urls)):
            now = datetime.datetime.now() 
            if self.urls[i]['start_datetime'] == None:
                url_index = i
                break
            else:
                start_datetime = strdatetime.datetime.now()
                if self.urls[i]['finish_datetime'] == "":
                    if now >= start_datetime + datetime.timedelta(hours=24):
                        url_index = i
                        break
                else:
                    if not 'frequency' in self.urls[i]:
                        freq = (24*60) # default to 24 hours
                    else:
                        freq = self.urls[i]['frequency']
                    if self.urls[i]['finish_datetime'] == None:
                        url_index = i
                        break
                    else:
                        finish_datetime = datetime.datetime.strptime(
                            self.urls[i]['finish_datetime'],
                            "%Y-%m-%d %H:%M:%S"
                        )
                        if now >= finish_datetime + datetime.timedelta(minutes=int(freq)):
                            url_index = i
                            break
        return url_index
             
    def start(self):
        self.broadcast_status()

    def broadcast_status(self):
        while not self.exiting:
            packet = {
                'urls': self.urls,
                'current_url_index': self.current_url_index,
                'scrapers': self.scrapers,
                'status_datetime': str(datetime.datetime.now()), 
            }
            self.bus_access.send_message(
                command = 'dispatcher_status',
                destination_id = 'broadcast',
                message = packet,
            )

            if not self.exiting:
                self.bus_access.tsleep(self.broadcast_interval)

    def send_url(self,url_index,destination_id):
        packet = self.urls[url_index]
        self.bus_access.send_message(
            command = 'url_dispatch',
            destination_id = destination_id,
            message = packet,
        )

    def get_remaining_url_count(self):
        remaining = (len(self.urls)-1) - (self.current_url_index-1) 
        return remaining

    def stop(self):
        self.exiting = True
        self.bus_access.stop_listening()

    def _reqcallback(self,payload):
        #print "Dispatcher._reqcallback(): handling callback"
        #try:
        if True:
            response = payload
            if response['command'] == 'set_dispatcher_urls':
                self.set_urls(response['message']['urls'])

            if response['command'] == 'scraper_finished':
                for i in range(0,len(self.urls)):
                    target_url = response['message']['url_data']['target_url']
                    source_id = response['source_id']
                    now = str(datetime.datetime.now())
                    if self.urls[i]['target_url'] == target_url and self.urls[i]['scraper_id'] == source_id:
                        self.urls[i]['finish_datetime'] = now

            if response['command'] == 'scraper_available':
                if self._DEBUG == True:
                    print "Dispatcher._reqcallback(): Scraper availability seen"
                if self.self_dispatch == True:
                    if self._DEBUG == True:
                        print "Dispatcher._reqcallback(): Self dispatching"
                    url_index = self.get_next_url_index()
                else:
                    if self._DEBUG == True:
                        print "Dispatcher._reqcallback(): Not self dispatching, getting durrent url index ..."
                        print "len(self.urls): {0}, self.current_url_index: {1}, len(self.urls)-1: {2}".format(
                            len(self.urls), self.current_url_index, len(self.urls)-1,
                        )
                    url_index = -1
                    if len(self.urls) != 0 and self.current_url_index <= len(self.urls)-1:
                        url_index = self.current_url_index
                        self.current_url_index+=1
                    else:
                        if self._DEBUG == True:
                            print "Dispatcher._reqcallback(): All queued URLs dispatched"
                if not url_index == -1:
                    if self._DEBUG == True:
                        print "Dispatcher._reqcallback(): Dispatching URL"
                    self.urls[url_index]['start_datetime'] = str(datetime.datetime.now())
                    self.urls[url_index]['scraper_id'] = response['source_id']
                    self.urls[url_index]['status'] = 'running'
                    self.send_url(url_index,response['source_id'])
                else:
                    if self._DEBUG == True:
                        print "Dispatcher._reqcallback(): No URLs to dispatch"

            if response['command'] == 'global_shutdown':
                if self._DEBUG == True:
                    print "Exiting."
                self.stop()
        #except Exception, e:
        #    print "BarkingOwl Dispatcher, ERROR: {0}".format(str(e))



import datetime
import time
import resource
import urllib2
import urlparse
from bs4 import BeautifulSoup
import tldextract
import requests
import magic
import json

VERSION = "v0.6.4.1"

class Scraper(object):

    def __init__(self,
                 file_header_size=1024,
                 max_bandwidth=-1,
                 max_memory=-1,
                 user_agent="BarkingOwl Scraper/%s" % VERSION,
                 DEBUG=False):

        self._reset_scraper()

        self.__file_header_size = file_header_size
        self.__max_bandwidth = max_bandwidth
        self.__max_memory = max_memory
        self.__user_agent = user_agent

        self._DEBUG = DEBUG

        self._start_callback = None
        self._finished_callback = None
        self._found_doc_callback = None
        self._new_url_callback = None
        self._bandwidth_limit_callback = None
        self._memory_limit_callback = None
        self._error_callback = None

    def set_url_data(self, url_data):
        required = [
            'target_url',
            'doc_types',
            'title',
            'description',
            'max_link_level',
            'creation_datetime',
            'allowed_domains',
            'sleep_time',
        ]
        for key in required:
            if key not in url_data:
                raise Exception("Invalid url_data.  %r is required." % key)

        self._data['url_data'] = url_data
        self._data_loaded = True

    def set_callbacks(self, start_callback=None, finished_callback=None, \
            found_doc_callback=None, new_url_callback=None, \
            bandwidth_limit_callback=None, memory_limit_callback=None, \
            error_callback=None):

        self._start_callback = start_callback
        self._finished_callback = finished_callback
        self._found_doc_callback = found_doc_callback
        self._new_url_callback = new_url_callback
        self._bandwidth_limit = bandwidth_limit_callback
        self._memory_limit = memory_limit_callback
        self._error_callback = error_callback

    def _start(self):
        if not self._start_callback == None:
            try:
                self._start_callback(self._data)
            except:
                pass

    def _stop(self):
        if not self._finished_callback == None:
            try:
                self._finished_callback(self._data)
            except:
                pass
    def _found_doc(self, document_url):
        if not self._found_doc_callback == None:
            try:
                self._found_doc_callback(self._data, document_url['url'])
            except Exception, e:
                print "Scraper._found_doc(): ERROR: {0}".format(e)

    def _new_url(self, url):
        if not self._new_url_callback == None:
            try:
                self._new_url_callback(self._data, url)
            except:
                pass

    def _bandwidth_limit(self):
        if not self._bandwidth_limit_callback == None:
            try:
                self._bandwidth_limit_callback(self._data)
            except:
                pass

    def _memory_limit(self):
        if not self._memory_limit_callback == None:
            try:
                self._memory_limit_callback(self._data)
            except:
                pass

    def _error(self, error_text):
        if( not self._error_callback == None ):
            try:
                self._error_callback(self._data, error_text)
            except:
                pass

    def _reset_scraper(self):
        self._data = {}
        self._data['seen_urls']  = []
        self._data['bandwidth'] = 0
        self._data['ignored_count'] = 0
        self._data['link_level'] = 1
        self._data['bad_urls'] = []
        self._data['documents'] = []
        self._data['file_header_size'] = 1024 
        self._data['max_type_try_count'] = 4
        self._data['url_data'] = {}
        self._data['working'] = False
        self._data['elapsed_time'] = None
        self._data['start_datetime'] = None
        self._data['stop_datetime'] = None 

        self._stopping = False
        self._data_loaded = False

    def stop(self):
        self._stopping = True
        
    def start(self):
        
        if self._DEBUG == True:
            print "Starting scraper ..."        

        if self._data_loaded == False:
            raise Exception("URL Data not set.")

        self._stopping = False
        self._data['working'] = True
        self._start()
        start_datetime = datetime.datetime.now() 
        success = self._find_documents()
        stop_datetime = datetime.datetime.now()
        self._data['working'] = False
        self._stop()
        self._data['elapsed_time'] = (stop_datetime - start_datetime)
        self._data['start_datetime'] = start_datetime
        self._data['stop_datetime'] = stop_datetime

        #print json.dumps(self._data, sort_keys=True,
        #    indent=4, separators=(',', ': '))

        return self._data

    def _find_documents(self):
        root_url = {
            'url': self._data['url_data']['target_url'],
            'tag_text': '<root>',
            'page_title': '<root>',
            'page_url': self._data['url_data']['target_url'],
        }
        success = self._process_urls([root_url])
        return success

    def _process_urls(self, urls):

        if self._stopping == True:
            return False

        if self._data['link_level'] > self._data['url_data']['max_link_level']:
            return False
        else:
            self._data['link_level'] += 1
            
        for url in urls:

            page_urls = self._get_page_urls(url)

            for page_url in page_urls:
                if self._stopping == True:
                    return False
                if not page_url in self._data['bad_urls'] and \
                        not any(seen_url['url'] == page_url['url'] \
                        for seen_url in self._data['seen_urls']):
                    doc_type = self._type_document(page_url)
                    if '*' in self._data['url_data']['doc_types'] or \
                            (doc_type != '' and \
                            doc_type in self._data['url_data']['doc_types']):
                        if not any(doc['url'] == page_url \
                                for doc in self._data['documents']):
                            new_doc = {
                                'url': page_url,
                                'found_datetime': str(datetime.datetime.now()),
                            }
                            self._data['documents'].append(new_doc)
                            self._found_doc(new_doc)
                else:
                    self._data['ignored_count'] += 1
            success = self._process_urls(page_urls)

        self._data['link_level'] -= 1

        return True
        
    def _get_page_urls(self, target_url):

        page_urls = []

        try:

            if self._data['url_data']['sleep_time'] > 0:
                time.sleep(self._data['url_data']['sleep_time'])

            response = ''
            try:
                headers = {
                    'User-Agent': '%s (getting urls)' % self.__user_agent,
                }
                response = requests.get(target_url['url'], headers=headers)
                self._data['bandwidth'] += len(response.headers)
                self._data['bandwidth'] += len(response.text)
            except:
                raise Exception("Bad URL")
                pass

            soup = BeautifulSoup(response.text)
            
            page_title = ''
            page_title_tag = soup.find_all('title')
            if len(page_title_tag) != 0 and page_title_tag[0].string != None:
                page_title = page_title_tag[0].string.strip()

            tag_types = [
                ('a','href'),
                ('img','src'),
                ('link','href'),
                ('object','data'),
                ('source','src'),
                ('script','src'),
                ('embed','src'),
                ('iframe','src'),
            ]
            
            for tag_type,verb in tag_types:
                tags = soup.find_all(tag_type)
                for tag in tags:
                    tag_text = ''
                    if len(tag.contents) >= 1:
                        if tag.string != None:
                            tag_text = tag.string.strip()
                        
                    raw_url = ''
                    try:
                        raw_url = tag[verb].encode('utf-8')
                    except:
                        continue
                    
                    full_url = urlparse.urljoin(target_url['url'], raw_url)

                    match = self._check_match(full_url)

                    if match == False:
                        continue

                    url = {
                        'url': full_url,
                        'tag_text': tag_text,
                        'page_title': page_title,
                        'page_url': target_url['url'],
                    }
                    page_urls.append(url)

        except:
            pass

        return page_urls

    def _check_match(self, url):
        match = False
        url_domain = tldextract.extract(url).domain.lower()
        target_url_domain = tldextract.extract(
            self._data['url_data']['target_url']
        ).domain.lower()
        if url_domain == target_url_domain or \
                url_domain in self._data['url_data']['allowed_domains']:
            match = True
        return match

    def _type_document(self, url):
        header_size = 0
        try_count = 0
        document_type = None
        for seen_url in self._data['seen_urls']:
            if url['url'] == seen_url:
                document_type = seen_url['type']
        if document_type == None and not url['url'] in self._data['bad_urls']:
            error_on_get = False
            while document_type == None and \
                    try_count < (self._data['max_type_try_count'] + 1):

                if self._data['url_data']['sleep_time'] > 0:
                    time.sleep(self._data['url_data']['sleep_time'])

                if try_count == self._data['max_type_try_count']:
                    # if we've tried the max number of times, then just downlaod
                    # the entire file to type it
                    req = urllib2.Request(url['url'])
                else:
                    header_size += self._data['file_header_size']
                    headers = {
                        'Range': 'byte=0-%i' % header_size,
                        'User-Agent': '%s (typing link)' % self.__user_agent,
                    }
                    req = urllib2.Request(url['url'], headers=headers)
                try:
                    open_url = urllib2.urlopen(req,timeout=5)
                    headers = open_url.info()
                    payload = open_url.read(
                        self._data['file_header_size']
                    )
                    self._data['bandwidth'] += ( len(headers) + len(payload) )
                    document_type = magic.from_buffer(payload, mime=True)
                except Exception, e:
                    # if this is our last try, and there was an error record the URL as 'bad'
                    if try_count == self._data['max_type_try_count']:
                        if not url['url'] in self._data['bad_urls']:
                            self._data['bad_urls'].append(url['url'])
                try_count += 1
        self._data['seen_urls'].append({
            'url': url['url'],
            'type': document_type,
        })
        self._new_url(url)
        if self._DEBUG == True:
            print "{0} : {1}".format(document_type, url['url'])
            print ("Scraper: Bandwidth: {1} Bytes, URL Count: {2}, " \
                "Document Count: {3}, Ignored Count: {4}, Try Count: {5}.".format(
                    resource.getrusage(resource.RUSAGE_SELF).ru_maxrss,
                    self._data['bandwidth'],
                    len(self._data['seen_urls']),
                    len(self._data['documents']),
                    self._data['ignored_count'],
                    try_count,
                ))
            print "\n"
        return document_type



import json
import datetime
from optparse import OptionParser
from scraper import Scraper

_DEBUG = False

def print_doc_info(_data, document_url):
    if _DEBUG == True:
        print "    {0} : {1}".format(document_url['tag_text'], document_url['url'])

if __name__ == '__main__':

    parser = OptionParser()
        
    parser.add_option("-u", "--target-url", dest="target_url",
        help="Target URL to scrape", metavar="URL")

    parser.add_option("-t", "--doc-type", dest="doc_type",
        help="Document type to look for.", metavar="DOCTYPE")

    parser.add_option("-l", "--max-link-level", dest="max_link_level",
        help="Maximum links to follow.", metavar="MAXLEVEL")

    parser.add_option("-j", "--json-output", action="store_true", 
        dest="json_output", help="Produce Pretty JSON output.", 
        default=False)

    (options, args) = parser.parse_args()

    if not options.target_url == '' and not options.target_url == None and \
            not options.doc_type == '' and not options.doc_type == None and \
            not options.max_link_level == '' and \
            not options.max_link_level == None and \
            not options.json_output == '' and not options.json_output == None:

        #DEBUG = False
        if options.json_output == False:
            _DEBUG = True

        if _DEBUG == True:
            print " -- CLI BarkingOwl Scraper -- "

        url = {
            'target_url': options.target_url,
            'doc_types': [
                options.doc_type,
            ],
            'title': options.target_url,
            'description': options.target_url,
            'max_link_level': int(options.max_link_level),
            'creation_datetime': str(datetime.datetime.now()),
            'allowed_domains': [
            ],
            'sleep_time': 0, # do not sleep between URL fetches
        }

        #try:
        if True:
            scraper = Scraper(DEBUG=_DEBUG)
            scraper.set_callbacks(
                found_doc_callback = print_doc_info,
            )
            scraper.set_url_data(url)

            if _DEBUG == True:
                print "\nStarting Scraper on {0} ...\n\n".format(options.target_url)
            data = scraper.start()
            if _DEBUG == True:
                print "\n\nScraper complete.\n"

            if _DEBUG == True:
                print "BarkingOwl Scraper found {0} documents on {1}.\n\n".format(
                    len(data['documents']),
                    options.target_url,
                )

            if options.json_output == True:
                data = scraper._data
                for key in data:
                    if isinstance(data[key], datetime.datetime) or \
                            isinstance(data[key], datetime.timedelta):
                        data[key] = str(data[key])
                print json.dumps(scraper._data, sort_keys=True,
                    indent=4, separators=(',', ': '))

        #except:
        #    if DEBUG == True:
        #        print "Yikes!  An error occured while the scraper was running.  Exiting."
        #    else:
        #        print '{"error_text": " An error occured while the scraper was running."}'
    else:
        print "Error: missing CLI arguments.  Try -h for help."



from .scraper import Scraper,ScraperWrapper
from .dispatcher import Dispatcher
from .globalshutdown import GlobalShutdown
from .busaccess import BusAccess
from pika import URLParameters as URLParameters



from setuptools import setup

long_description = \
'''

BarkingOwl is a scalable web crawler intended to be used to
find specific document types on websites, such as PDFs, XLS, 
TXT, HTML, etc.

For documentation checkout the github wiki:

https://github.com/thequbit/BarkingOwl/wiki

For a simple web frontend, check out barkingowl-frontend:

https://github.com/thequbit/barkingowl-frontend

'''

setup(
    name="BarkingOwl",
    version="0.6.5",
    license="GPL3",
    author="Timothy Duffy",
    author_email="tim@timduffy.me",
    packages=["barking_owl", "barking_owl.scraper", "barking_owl.dispatcher"],
    zip_safe=False,
    description='Scalable web crawler.',
    long_description=long_description,
    include_package_data=True,
    platforms="any",
    install_requires=[
      "beautifulsoup4==4.3.2",
      "libmagic==1.0",
      "pika==0.9.13",
      #"python-dateutil==2.2",
      "python-magic==0.4.6",
      "six==1.5.2",
      "wsgiref==0.1.2",
      "tldextract==1.5.1",
      "requests==2.5.1",
    ], 
    url="https://github.com/thequbit/BarkingOwl",
    classifiers=[
      'Development Status :: 4 - Beta',
      'Environment :: Console',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
      #'Operating System :: POSIX',
    ],
)
